import datetime

from brigid.core.entities import BaseEntity


class MetaInfo(BaseEntity):
    language: str
    allowed_languages: list[str]

    title: str
    description: str
    author: str
    tags: list[str]
    published_at: datetime.datetime | None

    seo_image_url: str | None



import pathlib

import pydantic
import pydantic_settings

from brigid.core.settings import BaseSettings


class PhotoSwipe(pydantic.BaseModel):
    stylesheet: str = "https://cdnjs.cloudflare.com/ajax/libs/photoswipe/5.4.2/photoswipe.min.css"
    lightbox: str = "https://cdnjs.cloudflare.com/ajax/libs/photoswipe/5.4.2/photoswipe-lightbox.esm.min.js"
    pswp: str = "https://cdnjs.cloudflare.com/ajax/libs/photoswipe/5.4.2/photoswipe.esm.min.js"


class Templates(pydantic.BaseModel):
    directory: pathlib.Path = pathlib.Path(__file__).parent / "templates"
    reload: bool = True


class Settings(BaseSettings):
    photoswipe: PhotoSwipe = PhotoSwipe()

    templates: Templates = Templates()

    model_config = pydantic_settings.SettingsConfigDict(env_prefix="BRIGID_THEME_")


settings = Settings()



from brigid.library.entities import Page
from brigid.renderer.markdown_render import render_page
from brigid.validation.entities import Error


def page_is_rendered(page: Page) -> list[Error]:
    context = render_page(page=page)

    errors = []

    for error in context.errors:
        errors.append(Error(filepath=page.path, message=f'Render error "{error.message}" in "{error.failed_text}"'))

    return errors



import markdown

from brigid.core import logging
from brigid.library.entities import Page
from brigid.library.storage import storage
from brigid.renderer.context import RenderContext, markdown_context, render_context
from brigid.renderer.processors.collection_block import CollectionBlockExtension
from brigid.renderer.processors.external_links import EXTERNAL_LINK_RE, ExternalLinkInlineProcessor
from brigid.renderer.processors.header_anchors import HeaderAnchorsExtension
from brigid.renderer.processors.images_block import ImagesBlockExtension
from brigid.renderer.processors.internal_links import INTERNAL_LINK_RE, InternalLinkInlineProcessor
from brigid.renderer.processors.old_image_markup_validation import IMAGE_LINK_RE, ImageInlineValidator
from brigid.renderer.processors.snippets import SnippetExtension
from brigid.renderer.processors.youtube_block import YouTubeBlockExtension

logger = logging.get_module_logger()

_renderers: list[markdown.Markdown] = []


def _construct() -> markdown.Markdown:

    configs = {
        "pymdownx.tilde": {"smart_delete": True, "subscript": True},
    }

    renderer = markdown.Markdown(
        extensions=[
            ImagesBlockExtension(),
            YouTubeBlockExtension(),
            HeaderAnchorsExtension(),
            CollectionBlockExtension(),
            SnippetExtension(dedent_subsections=True),
            "md_in_html",
            "pymdownx.betterem",
            "pymdownx.superfences",
            "pymdownx.saneheaders",
            "pymdownx.tilde",
            "pymdownx.blocks.details",
        ],
        extension_configs=configs,
    )

    # all images must be processed by ImagesBlockExtension
    # because it gives more control over the output and more validation
    renderer.inlinePatterns.deregister("image_link")
    renderer.inlinePatterns.deregister("image_reference")
    renderer.inlinePatterns.deregister("short_image_ref")

    # remove classic links processor
    renderer.inlinePatterns.deregister("link")

    # add modified processors
    renderer.inlinePatterns.register(ImageInlineValidator(IMAGE_LINK_RE, renderer), "image_link_validator", 160)
    renderer.inlinePatterns.register(ExternalLinkInlineProcessor(EXTERNAL_LINK_RE, renderer), "external_link", 160)
    renderer.inlinePatterns.register(InternalLinkInlineProcessor(INTERNAL_LINK_RE, renderer), "internal_link", 160)

    return renderer


def renderer(index: int) -> markdown.Markdown:
    while len(_renderers) <= index:
        _renderers.append(_construct())

    return _renderers[index]


def render(text: str) -> str:
    context = render_context.get()

    content = renderer(context.renderer).convert(text)

    if not context.errors:
        return content

    logger.warning("markdown_render_errors", errors_number=len(context.errors))

    return repr(context.errors[0])


def render_page(page: Page) -> RenderContext:
    # TODO: reload only optionally

    context = RenderContext(page=page, article=storage.get_article(id=page.article_id), renderer=0)

    with markdown_context(context):
        context.content = render(page.body)

    return context


def render_page_intro(page: Page) -> RenderContext:
    # TODO: reload only optionally

    context = RenderContext(page=page, article=storage.get_article(id=page.article_id), renderer=0)

    with markdown_context(context):
        context.content = render(page.intro)

    return context


def render_text(text: str) -> RenderContext:
    prev_context = render_context.get()

    if prev_context is None:
        raise NotImplementedError("render_text must be called within render context")

    context = RenderContext(
        page=prev_context.page,
        article=prev_context.article,
        renderer=prev_context.renderer + 1,
    )

    with markdown_context(context):
        context.content = render(text)

    return context



import asyncio
from typing import AsyncGenerator, Generator

import fastapi
import pytest
import pytest_asyncio

from brigid.application import application


@pytest.fixture(scope="session", autouse=True)
def event_loop() -> Generator[asyncio.AbstractEventLoop, asyncio.AbstractEventLoop, None]:
    loop = asyncio.get_event_loop_policy().new_event_loop()
    yield loop
    loop.close()


@pytest_asyncio.fixture(scope="session", autouse=True)
async def app() -> AsyncGenerator[fastapi.FastAPI, None]:
    async with application.with_app() as app:
        yield app



from typing import Any

from sentry_sdk import init as initialize_sentry
from sentry_sdk.integrations.fastapi import FastApiIntegration
from sentry_sdk.integrations.logging import LoggingIntegration
from sentry_sdk.integrations.starlette import StarletteIntegration


def improve_fingerprint(event: dict[str, Any], hint: dict[str, Any]) -> dict[str, Any]:
    if "exc_info" not in hint:
        return event

    _, e, _ = hint["exc_info"]

    return event


def before_send(event: dict[str, Any], hint: dict[str, Any]) -> dict[str, Any]:
    return improve_fingerprint(event, hint)


def initialize(
    dsn: str, sample_rate: float, enable_tracing: bool, traces_sample_rate: float, environment: str
) -> None:
    initialize_sentry(
        dsn=dsn,
        enable_tracing=enable_tracing,
        sample_rate=sample_rate,
        traces_sample_rate=traces_sample_rate,
        attach_stacktrace=True,
        before_send=before_send,
        environment=environment,
        include_source_context=True,
        integrations=[
            # disable default logging integration to use specialized structlog-sentry processor
            LoggingIntegration(event_level=None, level=None),
            StarletteIntegration(transaction_style="endpoint"),
            FastApiIntegration(transaction_style="endpoint"),
        ],
    )



translations = {
    "ru": {
        "read_more": "Далее",
        "no_posts_found": "Ничего не найдено.",
        "pagination_first_page": "к новым",
        "pagination_last_page": "к старым",
        "tags_filter_header": "Нашли постов:",
        "article_similar_header": "Похожее",
        "last_posts_header": "Последние посты",
        "similarity_score_tooltip": "Уровень похожести",
    },
    "en": {
        "read_more": "Read more",
        "no_posts_found": "Nothing found.",
        "pagination_first_page": "to new",
        "pagination_last_page": "to old",
        "tags_filter_header": "Found posts:",
        "article_similar_header": "Similar",
        "last_posts_header": "Last posts",
        "similarity_score_tooltip": "Similarity score",
    },
}



from typing import Any
from unittest.mock import MagicMock, patch

import pytest

from brigid.library.tests import make as library_make
from brigid.renderer.markdown_render import render_page
from brigid.renderer.processors.images_block import Image, ImageModel, ImagesBlock, ImagesModel


class TestImage:

    def test_escape_alt(self) -> None:
        image = Image(src="test.jpg", alt="complex <p>test</p> bla-bla <<<")

        assert image.alt == "complex test bla-bla <<<"


class TestImagesModel:

    def test_minimum_1_image(self) -> None:
        with pytest.raises(ValueError):
            ImagesModel(caption="test", images=[])

    def test_first_image_alt_from_caption(self) -> None:
        images = ImagesModel(caption="test_caption", images=[Image(src="test.jpg")])
        assert images.images[0].alt == "test_caption"

        images = ImagesModel(
            caption="test_caption", images=[Image(src="test.jpg"), Image(src="test2.jpg", alt="alt_2")]
        )
        assert images.images[0].alt == "test_caption"
        assert images.images[1].alt == "alt_2"

    def test_all_images_must_have_alts(self) -> None:
        with pytest.raises(ValueError):
            ImagesModel(caption="test_caption", images=[Image(src="test.jpg"), Image(src="test2.jpg")])

        ImagesModel(
            caption="test_caption", images=[Image(src="test.jpg", alt="alt_1"), Image(src="test2.jpg", alt="alt_2")]
        )

    def test_all_images_must_have_alts_besides_the_first_one(self) -> None:
        ImagesModel(caption="test_caption", images=[Image(src="test.jpg"), Image(src="test2.jpg", alt="alt_1")])

    def test_default_galery_class(self) -> None:
        images = ImagesModel(caption="test_caption", images=[Image(src="test.jpg")])
        assert images.galery_class == "brigid-images-1"

        images = ImagesModel(
            caption="test_caption", images=[Image(src="test.jpg", alt="alt_1"), Image(src="test2.jpg", alt="alt_2")]
        )
        assert images.galery_class == "brigid-images-2"

    def test_galery_class(self) -> None:
        images = ImagesModel(caption="test_caption", images=[Image(src="test.jpg")], galery_class="test-galery")
        assert images.galery_class == "test-galery"


class TestImageModel:

    def test_alt_or_caption(self) -> None:
        with pytest.raises(ValueError):
            ImageModel(src="test.jpg")

        ImageModel(src="test.jpg", alt="alt_1")

        image = ImageModel(src="test.jpg", caption="caption_1")
        assert image.alt == "caption_1"

        ImageModel(src="test.jpg", alt="alt_1", caption="caption_1")


class TestImagesBlock:

    def test_root_css_classes(self) -> None:
        images = ImagesModel(
            caption="test_caption", images=[Image(src="test.jpg", alt="alt_1"), Image(src="test2.jpg", alt="alt_2")]
        )

        images_block = ImagesBlock(length=3, tracker={}, block_mgr=MagicMock(), config={})

        assert set(images_block.root_css_classes(images)) == {"brigid-images", "brigid-images-2"}

    def test_process_data__images_model(self) -> None:
        images_block = ImagesBlock(length=3, tracker={}, block_mgr=MagicMock(), config={})

        images = ImagesModel(
            caption="test_caption", images=[Image(src="test.jpg", alt="alt_1"), Image(src="test2.jpg", alt="alt_2")]
        )

        assert images_block.process_data(images.replace()) == images

    def test_process_data__image_model(self) -> None:
        images_block = ImagesBlock(length=3, tracker={}, block_mgr=MagicMock(), config={})

        image = ImageModel(src="test.jpg", alt="alt_1", caption="test caption", galery_class="test-galery")

        images = ImagesModel(
            caption="test caption", galery_class="test-galery", images=[Image(src="test.jpg", alt="alt_1")]
        )

        assert images_block.process_data(image) == images

    def test_full_render(self) -> None:

        def render_in_theme(block: ImagesBlock, data: dict[str, Any]) -> str:
            assert data["current_article"] == article
            assert data["current_page"] == page
            assert data["data"] == ImagesModel(
                caption="some caption", images=[Image(src="./images/image-1.jpg", alt="some alt")]
            )
            return "rendered"

        text = """
/// brigid-images
src = "./images/image-1.jpg"
alt = "some alt"
caption = "some caption"
///
        """

        article = library_make.article()
        page = library_make.page(article, body=text)

        with patch("brigid.renderer.processors.images_block.ImagesBlock.render_in_theme", render_in_theme):
            result = render_page(page)

        assert result.page == page
        assert result.article == article
        assert result.renderer == 0
        assert result.errors == []
        assert result.content == '<figure class="brigid-images brigid-images-1">rendered</figure>'
